@@ -1,3 +1,5 @@
+import itertools
+import inspect
 import logging
 import os
 import shutil
@@ -14,6 +16,8 @@ from .engine import Engine
 from .feeds import RSSFeedEngine
 from .links import Link
 from .page import Page
+from .search import Search
+
 def get_subcollections(collection):
 
     subcollection_set = set() # creates a set to add new subcollections to
@@ -33,7 +37,6 @@ def get_subcollections(collection):
     return subcollection_set
 
 
-
 class Site:
     """
     The site stores your pages and collections to be rendered.
@@ -68,20 +71,21 @@ class Site:
             Relative and Absolute URLS
     """
 
-    routes: typing.List[str] = []
+    routes: typing.List[Page] = []
     output_path: Path = Path("output")
     static_path: Path = Path("static")
     SITE_TITLE: str = "Untitled Site"
     SITE_LINK: str = "https://example.com"
     SITE_URL: str = "https://example.com"
     strict: bool = False
+    default_engine: typing.Type[Engine] = Engine()
+    rss_engine: typing.Type[Engine] = RSSFeedEngine()
+    search: typing.Optional[typing.Type[Search]] = None
+    search_index_filename: str = 'search.json'
+    search_keys: typing.List[str] = []
+    timezone: str=''
 
-    def __init__(
-            self,
-            search=None,
-            search_keys=[],
-            timezone: str='',
-            ):
+    def __init__(self):
         """
         Clean Directory and Prepare Output Directory
 
@@ -112,9 +116,10 @@ class Site:
         if self.strict and self.output_path.exists():
             shutil.rmtree(self.output_path)
 
+        self.output_path.mkdir(exist_ok=True)
+
         # copy a defined static path into output path
         if Path(self.static_path).is_dir():
-            self.output_path.mkdir(exist_ok=True)
             shutil.copytree(
                 self.static_path,
                 self.output_path.joinpath(self.static_path),
@@ -123,23 +128,7 @@ class Site:
 
         # sets the timezone environment variable to the local timezone if not present
 
-        os.environ['render_engine_timezone'] = more_itertools.first_true(
-                [
-                    getattr(self, 'timezone', None),
-                    timezone,
-                    pendulum.local_timezone().name
-                    ]
-                )
-
-
-        self.engines: typing.Dict[str, typing.Type[Engine]] = {
-            "default_engine": Engine(),
-            "rss_engine": RSSFeedEngine(),
-        }
-
-        self.search = search
-        self.search_keys = search_keys
-        self.search_index_filename = "search.json"
+        os.environ['render_engine_timezone'] = self.timezone or pendulum.local_timezone().name
 
 
     def register_collection(self, collection_cls: typing.Type[Collection]) -> None:
@@ -187,13 +176,13 @@ class Site:
             for archive_page in subcollection.archive:
                 self.route(archive_page)
 
-        if hasattr(collection, "feeds"):
+        if collection.feeds:
             for feed in collection.feeds:
                 self.register_feed(feed=feed, collection=collection)
 
 
     def register_feed(self, feed, collection: Collection) -> None:
-        extension = self.engines["rss_engine"].extension
+        extension = self.rss_engine.extension
         _feed = feed()
         _feed.slug = collection.__class__.__name__.lower()
         _feed.items = [page.rss_feed_item for page in collection._pages]
@@ -213,29 +202,47 @@ class Site:
 
 
     def render(self, dry_run: bool = False) -> None:
-
         for page in self.routes:
-            engine = self.engines.get(page.engine, self.engines["default_engine"])
-            content = engine.render(page, content=page.markup,
-                    markup=page.markup, **vars(self))
+
+            if page.engine:
+                engine = page.engine
+
+            else:
+                engine = self.default_engine
+
+            logging.debug(f'{engine=}')
+
+            template_attrs = self.get_public_attributes(page)
+
+            print(template_attrs.get('github', None))
+
+            content = engine.render(page, **template_attrs)
 
             for route in page.routes:
-                logging.info(f"starting on {route=}")
+                logging.debug(f"starting on {route=}")
                 route = self.output_path.joinpath(route.strip("/"))
                 route.mkdir(exist_ok=True)
                 filename = Path(page.slug).with_suffix(engine.extension)
                 filepath = route.joinpath(filename)
-
-                if not dry_run:
-                    filepath.write_text(content)
-
-                else:
-                    print(f"{content} writes to {filepath}")
+                filepath.write_text(content)
 
         if self.search:
             search_pages = filter(lambda x: x.no_index == False, self.routes)
-            search_index = self.search.build_index(
+            self.search.build_index(
                 pages=search_pages,
                 keys=self.search_keys,
                 filepath=self.output_path.joinpath(self.search_index_filename),
             )
+
+
+    def get_public_attributes(self, cls):
+        site_filtered_attrs = itertools.filterfalse(lambda x:
+                x[0].startswith('__'), inspect.getmembers(self))
+        site_dict = {x: y for x, y in site_filtered_attrs}
+
+        cls_filtered_attrs = itertools.filterfalse(lambda x:
+                x[0].startswith('__'), inspect.getmembers(cls))
+
+        cls_dict = {x: y for x, y in cls_filtered_attrs}
+
+        return {**site_dict, **cls_dict}
